/*****************************************************************************
 * Copyright (C) 2008-2024
 *   
 * Michiel Broek <mbroek at mbse dot eu>
 *
 * This file is part of the mbsePi-apps
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * mbsePi-apps is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ThermFerm; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *****************************************************************************/

#include "thermferm.h"
#include "xutil.h"


char *xmalloc(size_t size)
{
    char *tmp;

    tmp = malloc(size);
    if (!tmp) 
	abort();
			            
    return tmp;
}



char *xstrcpy(char *src)
{
    char    *tmp;

    if (src == NULL) 
	return(NULL);
    tmp = xmalloc(strlen(src)+1);
    strcpy(tmp, src);
    return tmp;
}



char *xstrcat(char *src, char *add)
{
    char    *tmp;
    size_t  size = 0;

    if ((add == NULL) || (strlen(add) == 0))
	return src;
    if (src)
	size = strlen(src);
    size += strlen(add);
    tmp = xmalloc(size + 1);
    *tmp = '\0';
    if (src) {
	strcpy(tmp, src);
	free(src);
    }
    strcat(tmp, add);
    return tmp;
}


char *xjscomma(char *src, int *flag)
{
    char        *tmp;
    int         fl = *flag;

    *flag = 1;
    if (fl)
        tmp = xstrcat(src, (char *)",");
    else
        return src;
    return tmp;
}


char *xjsobjstr(char *src, char *name, char *val, bool comma)
{
    char        *tmp;

    if (comma)
        tmp = xstrcat(src, (char *)",\"");
    else
        tmp = xstrcat(src, (char *)"\"");
    tmp = xstrcat(tmp, name);
    tmp = xstrcat(tmp, (char *)"\":");

    if (val && strlen(val)) {
        tmp = xstrcat(tmp, (char *)"\"");
        tmp = xstrcat(tmp, val);
        tmp = xstrcat(tmp, (char *)"\"");
    } else {
        tmp = xstrcat(tmp, (char *)"null");
    }
    return tmp;
}


char *xjsobjint(char *src, char *name, int val, bool comma)
{
    char        *tmp, buf[65];

    sprintf(buf, "%d", val);
    if (comma)
        tmp = xstrcat(src, (char *)",\"");
    else
        tmp = xstrcat(src, (char *)"\"");
    tmp = xstrcat(tmp, name);
    tmp = xstrcat(tmp, (char *)"\":");
    tmp = xstrcat(tmp, buf);
    return tmp;
}


char *xjsobjfloat(char *src, char *name, double val, bool comma)
{
    char        *tmp, buf[65];

    sprintf(buf, "%.6f", val);
    if (comma)
        tmp = xstrcat(src, (char *)",\"");
    else
        tmp = xstrcat(src, (char *)"\"");
    tmp = xstrcat(tmp, name);
    tmp = xstrcat(tmp, (char *)"\":");
    tmp = xstrcat(tmp, buf);
    return tmp;
}


char *xjsobjbool(char *src, char *name, bool val, bool comma)
{
    char        *tmp;

    if (comma)
        tmp = xstrcat(src, (char *)",\"");
    else
        tmp = xstrcat(src, (char *)"\"");
    tmp = xstrcat(tmp, name);
    tmp = xstrcat(tmp, (char *)"\":");
    if (val)
        tmp = xstrcat(tmp, (char *)"true");
    else
        tmp = xstrcat(tmp, (char *)"false");
    return tmp;
}


