#ifndef XUTIL_H
#define	XUTIL_H

/**
 * @brief Safe memory allocation.
 * @param size The size in bytes.
 * @return The allocaated string.
 */
char *xmalloc(size_t);


/**
 * @brief Return allocated string.
 * @param src The string to allocate and copy.
 * @return The allocaated string.
 */
char *xstrcpy(char *);


/**
 * @brief Add a string to a string and return allocated result.
 * @param src The original source string.
 * @param add The string to add.
 * @return The allocated result of both strings.
 */
char *xstrcat(char *, char *);


/**
 * @brief Add comma to the string if flag is set.
 * @param src The original source string.
 * @param flag If 0, don't add a comma, else add one. The flag will
 *             always be changed to 1.
 * @return The allocated result with or without the comma.
 */
char *xjscomma(char *src, int *flag);


/**
 * @brief Add json name value string pair.
 * @param src The original source string.
 * @param name The name part of to add.
 * @param val The value string. If empty or NULL, the result will be "null"
 * @param comma Begin with a comma or not.
 * @return The allocated result with the name and value added.
 */
char *xjsobjstr(char *src, char *name, char *val, bool comma);


/**
 * @brief Add json name value integer pair.
 * @param src The original source string.
 * @param name The name part of to add.
 * @param val The integer value.
 * @param comma Begin with a comma or not.
 * @return The allocated result with the name and value added.
 */
char *xjsobjint(char *src, char *name, int val, bool comma);


/**
 * @brief Add json name value float with 6 decimals pair.
 * @param src The original source string.
 * @param name The name part of to add.
 * @param val The float value.
 * @param comma Begin with a comma or not.
 * @return The allocated result with the name and value added.
 */
char *xjsobjfloat(char *src, char *name, double val, bool comma);


/**
 * @brief Add json name value boolean pair.
 * @param src The original source string.
 * @param name The name part of to add.
 * @param val The boolean value.
 * @param comma Begin with a comma or not.
 * @return The allocated result with the name and true or false added.
 */
char *xjsobjbool(char *src, char *name, bool val, bool comma);


#endif
